packer {
  required_plugins {
    windows-update = {
      version = ">= 0.12.0"
      source  = "github.com/rgl/windows-update"
    }
  }
}

source "vsphere-iso" "win10" {
  vcenter_server          = var.vcenter_server
  username                = var.vsphere_username
  password                = var.vsphere_password
  insecure_connection     = "true"

  vm_name                 = "${var.vm_guestos}-${var.vm_name}"
  datacenter              = var.vsphere_datacenter
  host                    = var.vsphere_host
  folder                  = var.vsphere_folder
  datastore               = var.vsphere_datastore

  guest_os_type           = "windows9_64Guest"

  boot_order              = "disk,cdrom"
  boot_command            = [""]
  boot_wait               = "5m"

  communicator            = "winrm"
  winrm_username          = "administrator"
  winrm_password          = var.winrm_password
  winrm_timeout           = "10m"

  CPUs                    = 2
  RAM                     = 8192

  network_adapters {
    network               = var.vsphere_network
    network_card          = "vmxnet3"
  }
  storage {
    disk_size             = 20480
    disk_thin_provisioned = true
  }
  disk_controller_type    = ["lsilogic-sas"]
  usb_controller          = ["xhci"]

  floppy_files            = [
    "packer/preseed/Windows10/Autounattend.xml",
    "packer/preseed/Windows10/Sysprep_Unattend.xml",
    "scripts/Set-NetworkProfile.ps1",
    "scripts/Disable-WinRM.ps1",
    "scripts/Enable-WinRM.ps1",
    "scripts/Install-VMwareTools.cmd"
  ]
  iso_checksum            = "sha256:8D1663B71280533824CF95C7AB48ADAF5A187C38FCFF5B16A569F903688916D0"
  iso_paths               = [
    "ISO-files/VMware-tools-windows-11.3.5-18557794/VMware-tools-windows-11.3.5-18557794.iso"
  ]
  iso_url                 = "https://${var.repo_username}:${var.repo_password}@sn.itch.fyi/Repository/iso/Microsoft/Windows%2010/20H2/en_windows_10_enterprise_20H2_x64.iso"

  shutdown_command        = "C:\\Windows\\System32\\Sysprep\\sysprep.exe /generalize /oobe /unattend:A:\\Sysprep_Unattend.xml"
  shutdown_timeout        = "1h"

  export {
    images                = false
    output_directory      = "/scratch/win10"
  }
  remove_cdrom            = true
}

build {
  sources = ["source.vsphere-iso.win10"]

  provisioner "windows-update" {
    filters = [
      "exclude:$_.Title -like '*Preview*'",
      "include:$true"
    ]
  }

  provisioner "powershell" {
    inline = [
      "[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12",
      "Invoke-Expression ((New-Object Net.WebClient).DownloadString('https://chocolatey.org/install.ps1'))"
    ]
  }

  provisioner "powershell" {
    inline = [
      "choco config set --name=limit-output --value=LimitOutput",
      "choco install -y 7zip.install",
      "choco install -y sysinternals",
      "choco install -y firefox"
    ]
  }

  provisioner "windows-update" {
    filters = [
      "exclude:$_.Title -like '*Preview*'",
      "include:$true"
    ]
  }

  provisioner "powershell" {
    inline = [
      "New-Item -Path 'C:\\Payload\\Scripts' -ItemType 'Directory' -Force:$True -Confirm:$False"
    ]
  }

  provisioner "file" {
    destination = "C:\\Payload\\"
    source      = "scripts/Windows10/payload/"
  }

  provisioner "powershell" {
    scripts = [
      "scripts/Windows10/Register-ScheduledTask.ps1"
    ]
  }

  post-processor "shell-local" {
    inline = [
      "pwsh -command \"& scripts/Update-OvfConfiguration.ps1 \\",
      " -OVFFile '/scratch/win10/${var.vm_guestos}-${var.vm_name}.ovf' \\",
      " -Parameter @{'appliance.name'='${var.vm_guestos}';'appliance.version'='${var.vm_name}'}\"",
      "pwsh -file scripts/Update-Manifest.ps1 \\",
      " -ManifestFileName '/scratch/win10/${var.vm_guestos}-${var.vm_name}.mf'",
      "ovftool --acceptAllEulas --allowExtraConfig --overwrite \\",
      " '/scratch/win10/${var.vm_guestos}-${var.vm_name}.ovf' \\",
      " /output/Windows10.ova"
    ]
  }
}